#! /usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Name:        searchpanel.py
# Purpose:     
#
# Author:      d0c 54v4g3
#
# Created:     2005/23/07
# RCS-ID:      $Id: searchpanel.py,v 1.11 2005/09/16 07:41:46 Inigo Exp $
# Copyright:   (c) 2005
# Licence:     See LICENCES.TXT
# Updated and modified into ABC : roee88
# Updated and modified for ABC_OKC : Old King Cole
#-----------------------------------------------------------------------------

import wx, ConfigParser
import wx.lib.iewin_old
from urllib import quote
from os.path import join


class SearchPanel(wx.Panel):
    def __init__(self, parent, utility):
        wx.Panel.__init__(self, parent, -1, style = wx.TAB_TRAVERSAL | wx.BORDER_STATIC, size = (600, 600))
        self.parent = parent
        self.utility = utility
        self.search_file = join(self.utility.datapath, "search.lst")

        # Custom page set in btconfig
        self.search_entries = []

        self.LoadURLs()         
        self.html = HTMLPanel(self, utility, self.search_entries, self.OnToggleSettings)
        self.settings = SettingsPanel(self, utility, self.search_entries, self.html.enginecombo)
        self.settings.Hide()     # Hidden by default

        colsizer = wx.FlexGridSizer(2, 1, 0, 0)
        colsizer.AddGrowableRow(0)
        colsizer.AddGrowableCol(0)
        colsizer.Add(self.html, 1, wx.EXPAND)
        colsizer.Add(self.settings, 1, wx.EXPAND)
        self.SetAutoLayout(True)
        self.SetSizer(colsizer)
        colsizer.Fit(self)
        colsizer.SetSizeHints(self)

    def OnToggleSettings(self, event):
        # Hides/shows the search settings/config panel
        if self.settings.IsShown():
            self.settings.Hide()
            self.Layout()
        else:
            self.settings.Show()
            self.Layout()

    def SaveURLs(self):
        try:
            f = open(self.search_file, 'w+')
            f.writelines('\xef\xbb\xbf')
            for params in self.search_entries:
                f.writelines((params[0] + '|' + params[1] + '|' + params[2] + "\n").encode('utf_8'))
            f.close()
        except:
            try:
                f.close()
            except:
                pass
            print "ERROR: reading from search.lst file"

    def LoadURLs(self):
        try:
            f = open(self.search_file, 'r+')
            firstline = True
            while True:
                configline = f.readline()        
                if configline == '' or configline == "\n":
                    break
                if firstline:
                    if configline[:3] == '\xef\xbb\xbf':
                        self.encoding = 'utf_8'
                        # Skip BOM
                        configline = configline[3:]
                        if configline == '' or configline == "\n":
                            break
                    else:
                        self.encoding = wx.GetDefaultPyEncoding()
                    firstline = False
                self.search_entries.append(configline[:-1].decode(self.encoding).split('|'))
            f.close()
        except:
            try:
                f.close()
            except:
                pass
            print "ERROR: writing to search.lst file"
        self.search_entries.sort(lambda u, v: cmp(u[0].lower(), v[0].lower()) or cmp(u[0], v[0]))


class SettingsPanel(wx.Panel):
    def __init__(self, parent, utility, searchengines, enginecombo):
        wx.Panel.__init__(self, parent, -1)
        self.parent = parent
        self.engines = searchengines
        self.enginecombo = enginecombo
        self.searchchoices = []
        self.utility = utility
        self.localize = self.utility.lang.get

        colsizer = wx.BoxSizer(wx.VERTICAL)

        setsizer = wx.BoxSizer(wx.HORIZONTAL)

        # Data edit boxes
        datasizer = wx.FlexGridSizer(4, 2, 0, 0)
        datasizer.AddGrowableCol(1)

        self.enginelist = wx.ListCtrl(self, -1, size = (180, -1), style = wx.LC_REPORT | wx.LC_SINGLE_SEL)
        self.enginelist.InsertColumn(0, self.localize('rssengines'), width = 176)
        self.enginelist.Bind(wx.EVT_LEFT_DOWN, self.OnListBox)
        self.enginelist.Bind(wx.EVT_KEY_DOWN, self.onKeyInList)
        self.enginelist.Bind(wx.EVT_RIGHT_DOWN, self.onRightInList)
        self.enginelist.Bind(wx.EVT_RIGHT_DCLICK, self.onRightInList)

        setsizer.Add(self.enginelist, 0, wx.ALL | wx.EXPAND | wx.FIXED_MINSIZE, 2)

        self.oldselectedindex = self.selectedindex = -1
        self.populateEngineList()

        # Name of the torrent search engine
        datasizer.Add(wx.StaticText(self, -1, self.localize('searchname')), 0,
                      wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT | wx.FIXED_MINSIZE | wx.ALL, 2)

        self.enginename = wx.TextCtrl(self, -1, "")
        self.enginename.Bind(wx.EVT_CHAR, self.onUpdateEngineName)
        self.enginename.Bind(wx.EVT_TEXT, self.onUpdateEngineNameText)
        datasizer.Add(self.enginename, 0, wx.ALL | wx.EXPAND | wx.FIXED_MINSIZE, 2)

        # Search link for the search engine
        datasizer.Add(wx.StaticText(self, -1, self.localize('searchurl')), 0,
                      wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT | wx.FIXED_MINSIZE | wx.ALL, 2)

        self.enginelink = wx.TextCtrl(self, -1, "")
        self.enginelink.Bind(wx.EVT_CHAR, self.onUpdateEngineLink)
        datasizer.Add(self.enginelink, 0, wx.ALL | wx.EXPAND | wx.FIXED_MINSIZE, 2)

        # Delimiter will hold the str/char that seperates search terms for each engine
        datasizer.Add(wx.StaticText(self, -1, self.localize('searchdelimiter')), 0,
                      wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT | wx.FIXED_MINSIZE | wx.ALL, 2)

        self.enginedelimiter = wx.TextCtrl(self, -1, "")
        self.enginedelimiter.Bind(wx.EVT_CHAR, self.onUpdateEngineDelimiter)
        datasizer.Add(self.enginedelimiter, 0, wx.ALL | wx.EXPAND | wx.FIXED_MINSIZE, 2)

        datasizer.AddSpacer((-1, 30))

        setsizer.Add(datasizer, 1, wx.EXPAND)

        colsizer.Add(setsizer, 1, wx.EXPAND)

        newbtn = wx.Button(self, -1, self.localize('rssnew'))
        self.Bind(wx.EVT_BUTTON, self.OnNewEntry, newbtn)

        self.svbtn = wx.Button(self, -1, self.localize('rsssave'))
        self.Bind(wx.EVT_BUTTON, self.OnSaveEntry, self.svbtn)

        self.rmbtn = wx.Button(self, -1, self.localize('rssremove'))
        self.Bind(wx.EVT_BUTTON, self.OnRemoveEntry, self.rmbtn)

        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add(newbtn)
        buttonbox.Add(self.svbtn, 0, wx.LEFT, 20)
        buttonbox.Add(self.rmbtn, 0, wx.LEFT, 20)

        colsizer.Add(buttonbox, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.ALL, 3)

        self.SetAutoLayout(True)
        self.SetSizer(colsizer)
        colsizer.Fit(self)

        self.OnListBox()

    def setSaveButtonStatus(self, originalvalue, newvaluegetter):
        if self.svbtn.IsEnabled():
            if newvaluegetter() == originalvalue and not self.engineEdited():
                self.svbtn.Disable()
        elif newvaluegetter() != originalvalue:
            self.svbtn.Enable()

    def onUpdateEngineName(self, event):
        if self.utility.onWriteFilteredText(event):
            wx.CallAfter(self.setSaveButtonStatus, self.engine[0], self.enginename.GetValue)

    def onUpdateEngineLink(self, event):
        if self.utility.onWriteFilteredText(event):
            wx.CallAfter(self.setSaveButtonStatus, self.engine[1], self.enginelink.GetValue)

    def onUpdateEngineDelimiter(self, event):
        if self.utility.onWriteFilteredText(event):
            wx.CallAfter(self.setSaveButtonStatus, self.engine[2], self.enginedelimiter.GetValue)

    def onUpdateEngineNameText(self, event):
        # Catch exception to avoid silly assertion in wxPython 2.7.1.3
        try:
            self.enginelist.SetStringItem(self.selectedindex, 0, self.enginename.GetValue())
        except:
            pass

    def onRightInList(self, event):
        pass

    def onKeyInList(self, event):
        if not self.enginelist.GetItemCount():
            return
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_UP:
            self.OnListBox(index = max(0, self.selectedindex - 1))
        elif keycode == wx.WXK_DOWN:
            self.OnListBox(index = min(self.enginelist.GetItemCount() - 1, self.selectedindex + 1))
        elif keycode == wx.WXK_PRIOR:
            self.OnListBox(index = max(0, self.selectedindex - self.enginelist.GetCountPerPage()))
        elif keycode == wx.WXK_NEXT:
            self.OnListBox(index = min(self.enginelist.GetItemCount() - 1, self.selectedindex + self.enginelist.GetCountPerPage()))
        elif keycode == wx.WXK_HOME:
            self.OnListBox(index = 0)
        elif keycode == wx.WXK_END:
            self.OnListBox(index = self.enginelist.GetItemCount() - 1)

    def getIndexFromName(self, name, skipcurrent = False):
        # Find the index that exactly matches name (FindItem can't be used because it doesn't take into account the case)
        maxenginenb = len(self.engines)
        index = 0
        while index < maxenginenb and (self.engines[index][0] != name or (skipcurrent and index == self.selectedindex)):
            index += 1
        if index == maxenginenb:
            return -1
        return index

    def OnListBox(self, event = None, index = None):
        savingneeded = False
        dropnewentry = False
        if index is not None or event is not None:
            if index is None:
                clickedindex = self.enginelist.HitTest(wx.Point(event.m_x, event.m_y))[0]
            else:
                clickedindex = index
            if clickedindex == self.selectedindex:
                if event is not None:
                    event.Skip()
                return
            if clickedindex >= 0:
                clickedname = self.engines[clickedindex][0]
            if self.svbtn.IsEnabled():
                dlg = wx.MessageDialog(self, self.localize('rssenginesaveerror'),
                                       self.localize('rssenginesaveerror_short'),
                                       wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
                dlgresult = dlg.ShowModal()
                dlg.Destroy()
                if dlgresult == wx.ID_CANCEL:
                    return
                if dlgresult == wx.ID_YES:
                    savingneeded = True
                elif not self.engine[0]:
                    dropnewentry = True
                    self.OnRemoveEntry()

            if not dropnewentry:
                if not savingneeded:
                    if self.selectedindex >= 0:
                        self.enginelist.SetStringItem(self.selectedindex, 0, self.engine[0])
                elif not self.OnSaveEntry():
                    return

            self.oldselectedindex = self.selectedindex
            if clickedindex >= 0:
                self.selectedindex = self.getIndexFromName(clickedname)
            else:
                self.selectedindex = -1

        if self.selectedindex == -1:
            try:
                if event is not None:
                    event.Skip()
                # Disable save/remove buttons and other fields/values if nothing is selected
                self.rmbtn.Disable() 
                self.enginename.SetValue('')
                self.enginelink.SetValue('')
                self.enginedelimiter.SetValue('')
                self.enginename.Disable()
                self.enginelink.Disable()
                self.enginedelimiter.Disable()
            except:
                pass
        else:
            try:
                self.enginelist.SetFocus()
                self.enginelist.Select(self.selectedindex)
                self.enginelist.Focus(self.selectedindex)
                self.engine = self.engines[self.selectedindex]
                # Re-enable fields
                if self.oldselectedindex == -1:
                    self.enginename.Enable()
                    self.enginelink.Enable()
                    self.enginedelimiter.Enable()
                    self.rmbtn.Enable() 
                # Name
                self.enginename.SetValue(self.engine[0])
                # Link
                self.enginelink.SetValue(self.engine[1])
                # Delimiter
                self.enginedelimiter.SetValue(self.engine[2])
            except:
                pass
        self.svbtn.Disable()

    def populateEngineList(self):
        self.engines.sort(lambda u, v: cmp(u[0].lower(), v[0].lower()) or cmp(u[0], v[0]))
        self.enginelist.Freeze()
        self.enginelist.DeleteAllItems()
        curengine = self.enginecombo.GetValue()
        enginenames = [engine[0] for engine in self.engines]
        index = 0
        for enginename in enginenames:
            self.enginelist.InsertStringItem(index, enginename)
            index += 1
        self.enginecombo.SetItems(enginenames)
        self.enginecombo.SetValue(curengine)
        self.enginelist.Thaw()

    def OnNewEntry(self, event):
        savingneeded = False
        dropnewentry = False
        if self.svbtn.IsEnabled():
            dlg = wx.MessageDialog(self, self.localize('rssenginesaveerror'),
                                   self.localize('rssenginesaveerror_short'),
                                   wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION)
            dlgresult = dlg.ShowModal()
            dlg.Destroy()
            if dlgresult == wx.ID_CANCEL:
                return
            if dlgresult == wx.ID_YES:
                savingneeded = True
            elif not self.engine[0]:
                dropnewentry = True
                self.OnRemoveEntry()

        if not dropnewentry:
            if not savingneeded:
                if self.selectedindex >= 0:
                    self.enginelist.SetStringItem(self.selectedindex, 0, self.engine[0])
            elif not self.OnSaveEntry():
                return

        self.engine = ['', 'http://', '+']
        self.engines.append(self.engine)
        self.oldselectedindex = self.selectedindex
        self.selectedindex = self.enginelist.GetItemCount()
        self.enginelist.InsertStringItem(self.selectedindex, '')
        self.enginelist.Select(self.selectedindex)
        self.enginelist.Focus(self.selectedindex)
        self.OnListBox()
        self.enginename.SetFocus()
        self.svbtn.Enable()

    def engineEdited(self):
        if not self.engine[0]:
            return True
        if self.enginename.GetValue() == self.engine[0] and self.enginelink.GetValue() == self.engine[1] \
           and self.enginedelimiter.GetValue() == self.engine[2]:
            return False
        return True

    def OnSaveEntry(self, event = None):
        newname = self.enginename.GetValue()

        # Name must not be blank
        if newname == '':
            dlg = wx.MessageDialog(self, self.localize('errorenginename'),
                                   self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            self.enginename.SetFocus()
            return False

        # Check if name already exists
        if self.getIndexFromName(newname, skipcurrent = True) != -1:
            dlg = wx.MessageDialog(self, self.localize('errorengineexists'),
                                   self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            self.enginename.SetFocus()
            self.enginename.SetSelection(-1, -1)
            return False

        self.engine = self.engines[self.selectedindex] = [newname, self.enginelink.GetValue(), self.enginedelimiter.GetValue()]
        self.populateEngineList()
        self.selectedindex = self.getIndexFromName(newname)
        if event is not None:
            self.enginelist.Select(self.selectedindex)
        self.parent.SaveURLs()

        self.svbtn.Disable()

        return True

    def OnRemoveEntry(self, event = None):
        if self.selectedindex >= 0:
            enginename = self.engines[self.selectedindex][0]
            if enginename:
                self.enginecombo.Delete(self.enginecombo.FindString(enginename))
            del self.engines[self.selectedindex]
            self.enginelist.DeleteItem(self.selectedindex)
            self.selectedindex = -1
            self.OnListBox()
            self.parent.SaveURLs()


class HTMLPanel(wx.Panel):
    def __init__(self, parent, utility, search_entries, togglesettingsfunc):
        wx.Panel.__init__(self, parent, -1, style = wx.TAB_TRAVERSAL | wx.NO_FULL_REPAINT_ON_RESIZE, size = (600, 600))
        self.utility = utility
        self.localize = self.utility.lang.get
        self.parent = parent
        self.OnToggleSettings = togglesettingsfunc
        self.search_entries = search_entries
        self.search_choices = []
        self.displayurl = True

        # Setup activex control
        self.iepanel = wx.Panel(self, -1, style = wx.TAB_TRAVERSAL | wx.NO_FULL_REPAINT_ON_RESIZE)
        self.ie = wx.lib.iewin_old.IEHtmlWindow(self.iepanel, -1)
        iepanelsizer = wx.BoxSizer(wx.VERTICAL)
        iepanelsizer.Add(self.ie, 1, wx.EXPAND)
        self.iepanel.SetSizer(iepanelsizer)
        self.iepanel.SetAutoLayout(True)

        colsizer = wx.BoxSizer(wx.VERTICAL)

        searchsizer = wx.FlexGridSizer(1, 8, 0, 0)
        searchsizer.AddGrowableCol(2)

        tbstyle = wx.TB_HORIZONTAL | wx.TB_TEXT | wx.TB_NODIVIDER | wx.NO_BORDER
        if self.utility.abcparams['flattopbtn'] == "1":
            tbstyle |= wx.TB_FLAT
        self.searchtoolbar = wx.ToolBar(self, -1, style = tbstyle)
        bitmap1 = self.utility.makeBitmap('srch_home.bmp')
        self.searchtoolbar.SetToolBitmapSize(wx.Size(bitmap1.GetWidth(), bitmap1.GetHeight()))
        id = wx.NewId()
        self.searchtoolbar.AddSimpleTool(id, bitmap1, self.localize('srch_home_short'), self.localize('srch_home_long'))
        self.Bind(wx.EVT_TOOL, self.OnHomeButton, id = id)
        id = wx.NewId()
        self.searchtoolbar.AddSimpleTool(id, self.utility.makeBitmap('srch_back.bmp'),
                                         self.localize('srch_back_short'), self.localize('srch_back_long'))
        self.Bind(wx.EVT_TOOL, self.OnPrevPageButton, id = id)
        id = wx.NewId()
        self.searchtoolbar.AddSimpleTool(id, self.utility.makeBitmap('srch_fwd.bmp'),
                                         self.localize('srch_fwd_short'), self.localize('srch_fwd_long'))
        self.Bind(wx.EVT_TOOL, self.OnNextPageButton, id = id)
        id = wx.NewId()
        self.searchtoolbar.AddSimpleTool(id, self.utility.makeBitmap('srch_stop.bmp'),
                                         self.localize('srch_stop_short'), self.localize('srch_stop_long'))
        self.Bind(wx.EVT_TOOL, self.OnStopButton, id = id)
        id = wx.NewId()
        self.searchtoolbar.AddSimpleTool(id, self.utility.makeBitmap('srch_reload.bmp'),
                                         self.localize('srch_reload_short'), self.localize('srch_reload_long'))
        self.Bind(wx.EVT_TOOL, self.OnRefreshPageButton, id = id)
        self.searchtoolbar.AddSeparator()
        self.searchtoolbar.Realize()

        searchsizer.Add(self.searchtoolbar, 0, wx.LEFT, 2)

        searchsizer.Add(wx.StaticText(self, -1, self.localize('searchSearch')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)

        self.searchterm = wx.TextCtrl(self, -1, "")
        self.searchterm.Bind(wx.EVT_CHAR, self.EvtChar)
        searchsizer.Add(self.searchterm, 1, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.BOTTOM | wx.RIGHT, 5)

        btn = wx.Button(self, -1, self.localize('searchFind'), style = wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.OnFindButton, btn)
        searchsizer.Add(btn, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 2)

        btn = wx.Button(self, -1, self.localize('searchGo'), style = wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.OnGoButton, btn)
        searchsizer.Add(btn, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 2)

        searchsizer.Add(wx.StaticText(self, -1, self.localize('searchEngine')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        self.enginecombo = wx.ComboBox(self, -1, "", size = (100, -1), choices = self.search_choices, style = wx.CB_DROPDOWN | wx.CB_READONLY)
        searchsizer.Add(self.enginecombo, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)

        self.populate_combo()

        if self.search_choices:
            self.enginecombo.SetSelection(0)

        tbstyle = wx.TB_HORIZONTAL | wx.TB_TEXT | wx.TB_NODIVIDER | wx.NO_BORDER
        if self.utility.abcparams['flattopbtn'] == "1":
            tbstyle |= wx.TB_FLAT
        self.configtoolbar = wx.ToolBar(self, -1, style = tbstyle)
        bitmap1 = self.utility.makeBitmap('srch_config.bmp')
        self.configtoolbar.SetToolBitmapSize(wx.Size(bitmap1.GetWidth(), bitmap1.GetHeight()))
        id = wx.NewId()
        self.configtoolbar.AddSimpleTool(id, bitmap1, self.localize('srch_config_short'), self.localize('srch_config_long'))
        self.Bind(wx.EVT_TOOL, self.OnToggleSettings, id = id)
        self.configtoolbar.Realize()

        searchsizer.Add(self.configtoolbar, 0, wx.RIGHT, 2)

        colsizer.Add(searchsizer, 0, wx.EXPAND)
        if self.utility.abcparams['flattopbtn'] == "1":
            colsizer.Add(self.iepanel, 1, wx.EXPAND)
        else:
            colsizer.Add(self.iepanel, 1, wx.EXPAND | wx.TOP, 3)

        self.SetSizer(colsizer)
        self.SetAutoLayout(True)

        self.ie.Bind(wx.lib.iewin_old.EVT_NewWindow2, self.OnNewWindow2)
        self.ie.Bind(wx.lib.iewin_old.EVT_StatusTextChange, self.UpdateStatusText)
        self.ie.Bind(wx.lib.iewin_old.EVT_DocumentComplete, self.OnDocumentComplete)

        if self.utility.abcparams["starthome"] == '1':
            self.displayHomePage()

    def setToolbarStyle(self, style):
        self.searchtoolbar.SetWindowStyle(style)
        self.configtoolbar.SetWindowStyle(style)
        self.searchtoolbar.Refresh()
        self.configtoolbar.Refresh()
        self.Layout()

    def UpdateStatusText(self, event):
        self.parent.parent.parent.SetStatusText(event.Text)

    def OnDocumentComplete(self, event):
        if self.displayurl:
            self.searchterm.SetValue(event.URL)

    def displayHomePage(self):
        cust_search_home = self.utility.abcparams['homepage']
        if cust_search_home != "":
            self.ie.Navigate2(cust_search_home)
        elif self.search_entries:        
            self.ie.Navigate2(self.search_entries[0][1])
        else:
            self.ie.Navigate2("about:blank")

    def OnHomeButton(self, event = None):
        self.displayurl = True
        self.displayHomePage()

    def OnPrevPageButton(self, event = None):
        self.displayurl = True
        try:
            self.ie.GoBack()
        except:
            pass

    def OnNextPageButton(self, event = None):
        self.displayurl = True
        try:
            self.ie.GoForward()
        except:
            pass

    def OnStopButton(self, event = None):
        self.displayurl = True
        try:
            self.ie.Stop()
        except:
            pass

    def OnGoButton(self, event = None, link = ''):
        self.displayurl = True
        if event is not None:
            link = self.searchterm.GetValue()
        if link != "":
            try:
                self.ie.Navigate2(link)
            except:
                pass
        self.iepanel.SetFocus()

    def OnFindButton(self, event = None):
        self.displayurl = False
        item = self.enginecombo.GetSelection()
        if item >= 0:
            try:
                self.ie.Navigate2(self.search_entries[item][1] + quote(self.searchterm.GetValue()).replace(" ", self.search_entries[item][2]))
            except:
                pass
        self.iepanel.SetFocus()

    def OnRefreshPageButton(self, event = None):
        self.displayurl = True
        try:
            self.ie.RefreshPage()
        except:
            pass

    def OnNewWindow2(self, event):
        event.Cancel = True

    def populate_combo(self):
        self.search_choices = []
        self.enginecombo.SetItems([param[0] for param in self.search_entries])
        self.enginecombo.SetSelection(0)

    def EvtChar(self, event): 
        if event.GetKeyCode() == 13:      # Search when return key pressed in search textctrl
            link = self.searchterm.GetValue().lower()
            if link.startswith("http://") or link.startswith("www."):
                self.OnGoButton()
            else:
                self.OnFindButton()
        event.Skip()
