##############################################################################
# Module : abctorrent.py
# Author : Old King Cole
# Date   : 15 October 2005
#
# Description : Torrent
#
##############################################################################
import sys, wx

from os import remove, listdir, rmdir, renames
from os.path import join, exists, normpath, split, commonprefix, splitdrive, getsize, isdir
from time import clock, time, localtime, strftime
from types import FloatType
from threading import Event

from win32file import GetDiskFreeSpaceEx

from utility import FileTree
from Dialogs.filenamescheckresultdlg import FileNamesCheckResultDialog
from BitTornado.bencode import bdecode

sysencoding = sys.getfilesystemencoding()


class ABCTorrent:
    def __init__(self, parent, src, dest, singlefile, status, priority, name,
                 maxupload, maxlocaluploadrate, maxlocaldownloadrate, uploadoption,
                 uploadtimeh, uploadtimem, uploadratio, downsize, upsize, totalsize,
                 infohash, progress, seedingtime, totalseedingtime, filesnumber,
                 label, shortlabel, timeoutswitch, timeoutaction, timeoutwork,
                 timeoutworkdown, timeoutworktrack, timeoutseed, timeoutseedup,
                 timeoutseedtrack, timeouttracker, timeoutdownload, timeoutupload,
                 movefolder, exttracker, exttrackerurl, termevt, defaulttracker,
                 private, multitracker, magnet, magnetname):
        self.parent = parent
        self.utility = parent.utility
        self.localize = self.utility.lang.get
        self.abcparams = self.utility.abcparams
        self.abcengine = None
        self.src = src
        self.dest = dest
        self.singlefile = singlefile
        self.status = status
        self.priority = priority
        self.name = name
        self.maxupload = maxupload
        self.maxuploadrate = 0.
        self.maxlocaluploadrate = maxlocaluploadrate
        self.maxdownloadrate = 0.
        self.maxlocaldownloadrate = maxlocaldownloadrate
        self.uploadoption = uploadoption[-1]
        self.uploadtillcopycomplete = (int(uploadoption) / 10 == 1)
        self.uploadtimeh = uploadtimeh
        self.uploadtimem = uploadtimem
        self.uploadtimemax = float(int(uploadtimeh) * 3600 + int(uploadtimem) * 60)
        self.uploadratio = uploadratio
        self.detailwin = None
        self.infowin = None
        self.downsize = downsize
        self.upsize = upsize
        # Torrent old status
        self.oldstatus = ""
        # Torrent old status before checking
        self.oldstatusbeforecheck = ""
        self.totalsize = totalsize
        self.requestedsize = float(totalsize)
        self.infohash = infohash
        self.progress = float(progress)
        self.complete = (progress == "100")
        self.checkedonce = False
        self.scrapedonce = False
        self.seedingtime = seedingtime
        self.totalseedingtime = totalseedingtime
        self.initSeedingTimeLeft()
        self.filesnumber = filesnumber
        self.filespriority = None
        self.filesname = None
        self.label = label
        self.shortlabel = shortlabel
        self.timeoutswitch = timeoutswitch
        self.timeoutaction = timeoutaction
        self.timeoutwork = timeoutwork
        self.timeoutworkdown = timeoutworkdown
        self.timeoutworktrack = timeoutworktrack
        self.timeoutseed = timeoutseed
        self.timeoutseedup = timeoutseedup
        self.timeoutseedtrack = timeoutseedtrack
        self.timeouttracker = timeouttracker
        self.timeoutdownload = timeoutdownload
        self.timeoutupload = timeoutupload
        self.movefolder = movefolder
        self.activity = 0
        self.unwantedfiles = 0
        self.scraping = False
        self.scrapeisfresh = False
        self.exttracker = exttracker
        self.exttrackerurl = exttrackerurl
        self.filesprogress = None
        self.lastgetscrape = 0
        if self.status == 'standby':
            self.sbstart = self.standbylastscrape = self.lastabsactivity = clock()
        else:
            self.sbstart = self.standbylastscrape = self.lastabsactivity = 0
        self.termevt = self.prevtermevt = termevt
        self.totallyallocated = False
        self.defaulttracker = defaulttracker
        self.private = private
        self.multitracker = multitracker
        self.magnet = magnet
        self.magnetname = magnetname
        # List of status for each file name inside torrent, built from 4 bits :
        # 1 : char not fixed ;
        # 2 : too long name in path
        # 3 : too long path
        # 4 : too long destination path
        # None if all files are OK
        self.filenamestatus = None

        # # Seeds and peers seen by tracker and number of complete copies
        self.seed = self.peer = self.utility.notrackeranswermarker
        self.numcopy = '?'
        self.message = ''
        # Flag set when BitTornado thread is finished
        self.shutdownflag = Event()
        # Timer for delayed resume when move data option is set to 'When completed'
        self.delayedresumetimer = None

        # True if torrent is moving its data
        self.ismovingdata = False

        # True if torrent is being deleted
        self.isbeingdeleted = False

    def displayStatusOnly(self, index = None, status = None):
        if self.ismovingdata:
            return
        if index is None:
            try:
                index = self.parent.proctab.index(self)
            except:
                return
        if status is None:
            status = self.status
        localizedstatus = self.localize(status)
        rank = self.parent.guiman.getRankfromID(6)
        if rank != -1:
            self.parent.list.SetStringItem(index, rank, localizedstatus)
        if status == 'finished':
            for rank in [self.parent.guiman.getRankfromID(colid) for colid in [8, 11, 14, 15, 16, 17, 20]]:
                if rank != -1:
                    self.parent.list.SetStringItem(index, rank, "")
            self.seed = self.peer = self.utility.notrackeranswermarker
            self.numcopy = '?'
        if self.infowin is not None:
            self.infowin.status.SetLabel(localizedstatus)
            if status == 'finished':
                self.infowin.uprate.SetLabel('0.0 ' + self.utility.kbps)
                self.infowin.colour.SetBackgroundColour(self.utility.colnoeng)
                self.infowin.colour.Refresh()
        if self.detailwin is not None:
            self.detailwin.status.SetLabel(localizedstatus)
            if status == 'finished':
                self.detailwin.init()
                self.detailwin.colour.SetBackgroundColour(self.utility.colnoeng)
                self.detailwin.colour.Refresh()

    def clearMessage(self, index, rank = None):
        self.displayMessage('', index, rank = rank)

    def displayMessage(self, msg, index = None, oldmsg = None, rank = None, keeptermevtmsg = None, msgindetails = None):
        if index is None:
            try:
                index = self.parent.proctab.index(self)
            except:
                return
        if rank is None:
            rank = self.parent.guiman.getRankfromID(13)
        if keeptermevtmsg is None:
            keeptermevtmsg = (self.abcparams['keeptermevtmsg'] == '1')
        if not msg and keeptermevtmsg:
            msg = self.termEvtMsg()

        if oldmsg is None or self.message[self.utility.timestamplen:] == oldmsg:
        # If oldmsg is not None, display msg only if current message ends with oldmsg (without time stamp)
            self.message = msg
            if rank != -1:
                self.parent.list.SetStringItem(index, rank, msg)
            if self.infowin is not None:
                if msgindetails is None:
                    msgindetails = (self.abcparams['msgindetails'] == "1")
                if msgindetails:
                    if msg:
                        self.infowin.showMessage()
                    else:
                        self.infowin.hideMessage()

    def checkComplete(self):
        # Check if all wanted files of a multifiles torrent are done
        if self.abcengine and not self.abcengine.onlycheck:
            self.abcengine.reportProgress()
        for index in xrange(self.filesnumber):
            if (self.filespriority is None or self.filespriority[index] != -1) \
               and (self.filesprogress is None or self.filesprogress[index] != 'Done'):
                if self.abcengine:
                    self.abcengine.btcomplete = False
                self.complete = False
                return
        if self.abcengine:
            self.abcengine.btcomplete = True
        else:
            self.complete = True

    def setActivity(self, activity, index = None, settimersonnextcheck = False):
        if activity == 1:
            if self.activity != 2 and self.abcengine:
                if settimersonnextcheck:
                    self.abcengine.timeractivityon = 0
                elif self.abcengine.timeractivityon:
                    self.abcengine.timeractivityon = clock()
                if (self.abcparams['trigwhenfinishseed'] == '1' or self.status != 'completed'
                    or self.abcparams['dontcheckseedactivity'] == '0') and self.status != 'onhold':
                    self.parent.torrentinactive -= 1
                    if self.parent.extratobestoppedmode:
                        if self.parent.torrentinactive < self.parent.activitymax:
                            self.parent.extratobestopped += 1
                        self.parent.freshactivetorrents.append(self)
            self.activity = 1
        elif activity == 2:
            self.activity = 2
        else:
            if self.abcengine:
                if settimersonnextcheck:
                    self.abcengine.timeractivityoff = 0
                elif self.abcengine.timeractivityoff:
                    self.abcengine.timeractivityoff = clock()
                if (self.abcparams['trigwhenfinishseed'] == '1' or self.status != 'completed'
                   or self.abcparams['dontcheckseedactivity'] == '0') and self.status != 'onhold':
                    self.parent.torrentinactive += 1
                    if self.parent.extratobestoppedmode:
                        self.parent.extratobestopped = max(0, self.parent.extratobestopped - 1)
            self.activity = 0
        rank = self.parent.guiman.getRankfromID(25)
        if rank != -1:
            if index == None:
                index = self.parent.proctab.index(self)
            self.parent.list.SetStringItem(index, rank, self.activityMarker())

    def formatedProgress(self, progress = None):
        if progress is None:
            progress = self.progress
        if progress == -1:
            return '  ?  %' + self.partialMarker()
        progressstring = str(.1 * int(10 * progress))
        if progressstring == '100.0':
            return '100%' + self.partialMarker()
        return progressstring + '%' + self.partialMarker()

    def isCheckingOrAllocating(self):
        return (self.abcengine
                and (self.abcengine.btstatus == "checkingexistingdata"
                or self.abcengine.btstatus == "allocatingdiskspace"
                or self.abcengine.btstatus == "movingdata"))

    def isWaiting(self):
        return (self.abcengine and self.abcengine.btstatus == "waiting")

    def isBlockedWaiting(self):
        if not self.abcengine:
            return False
        if not self.abcengine.dow:
            return True
        return self.abcengine.dow.waiting

    def useLocalUploadRate(self):
        return self.maxlocaluploadrate != 0

    def useLocalDownloadRate(self):
        return self.maxlocaldownloadrate != 0

    def writeFilePriorities(self):
        try:
            prioritiesfile = open(self.src + '.pri', "w")
            prioritiesfile.writelines([str(p) + '\n' for p in self.filespriority])
            prioritiesfile.close()
        except:
            pass

    def readFilePriorities(self, filesinfo):
        try:
            prioritiesfile = open(self.src + '.pri', "r")
        except:
            self.filespriority = None
            self.unwantedfiles = 0
        else:
            self.filespriority = []
            i = 0
            while True:
                templine = prioritiesfile.readline()
                if templine == "" or templine == "\n":
                    break
                else:
                    try:
                        priority = int(templine)
                    except:
                        # Corrupted priorities file
                        prioritiesfile.close()
                        self.filespriority = None
                        self.unwantedfiles = 0
                        remove(self.src + '.pri')
                        return
                    self.filespriority.append(priority)
                    if priority == -1:
                        self.unwantedfiles += 1
                        self.requestedsize -= filesinfo[i]['length']
                i += 1
            prioritiesfile.close()
            if len(self.filespriority) != self.filesnumber:
                # Corrupted priorities file
                self.filespriority = None
                self.unwantedfiles = 0
                remove(self.src + '.pri')

    def writeFileNameStatus(self):
        try:
            fnsfile = open(self.src + '.fns', "w")
            fnsfile.writelines([str(v) + '\n' for v in self.filenamestatus])
            fnsfile.close()
        except:
            pass

    def readFileNameStatus(self):
        try:
            fnsfile = open(self.src + '.fns', "r")
        except:
            self.filenamestatus = None
        else:
            self.filenamestatus = []
            i = 0
            while True:
                templine = fnsfile.readline()
                if templine == "" or templine == "\n":
                    break
                else:
                    try:
                        validity = int(templine)
                    except:
                        # Corrupted validities file
                        fnsfile.close()
                        self.filenamestatus = None
                        remove(self.src + '.fns')
                        return
                    self.filenamestatus.append(validity)
                i += 1
            fnsfile.close()
            if len(self.filenamestatus) != self.filesnumber:
                # Corrupted validities file
                self.filenamestatus = None
                remove(self.src + '.fns')

    def writeFileNames(self):
        try:
            namesfile = open(self.src + '.nam', "w")
            namesfile.writelines([n.encode('utf_8') + '\n' for n in self.filesname])
            namesfile.close()
        except:
            pass

    def readFileNames(self, filesinfo):
        try:
            namesfile = open(self.src + '.nam', "r")
        except:
            self.filesname = None
        else:
            self.filesname = []
            while True:
                templine = namesfile.readline()
                if templine == "" or templine == "\n":
                    break
                else:
                    try:
                        self.filesname.append(templine[:-1].decode('utf_8'))
                    except:
                        # Corrupted names file
                        namesfile.close()
                        self.filesname = None
                        remove(self.src + '.nam')
                        return
            namesfile.close()
            if len(self.filesname) != self.filesnumber:
                # Corrupted names file
                self.filesname = None
                remove(self.src + '.nam')

    def changeFilePriorities(self, newpriorities, index, filesinfo = None):
        if filesinfo is None:
            filesinfo = self.getResponse()['info']['files']
        updatenever = False
        if self.filespriority is None:
            self.filespriority = self.filesnumber * [1]

        # Update new priorities
        for p in newpriorities:
            if not updatenever and (self.filespriority[p[0]] == -1 or p[1] == -1):
                updatenever = True
            self.filespriority[p[0]] = p[1]
            if self.infowin is not None:
                self.infowin.updateFilePriority(p)
        if self.abcengine and self.abcengine.dow:
            self.abcengine.dow.setPriorities(self.filespriority)

        if updatenever:
            # If pool of "Never download" files has changed
            self.requestedsize = 0.
            self.unwantedfiles = 0
            downedbynotneversize = 0.
            for i in xrange(self.filesnumber):
                if self.filespriority[i] == -1:
                    self.unwantedfiles += 1
                else:
                    self.requestedsize += filesinfo[i]['length']
                    if self.filesprogress is not None:
                        fileprogressstr = self.filesprogress[i]
                        if fileprogressstr == 'Done':
                            fileprogress = 100.
                        else:
                            fileprogress = float(fileprogressstr)
                    else:
                        fileprogress = 0.
                    downedbynotneversize += filesinfo[i]['length'] * fileprogress / 100.

            sizerank = self.parent.guiman.getRankfromID(9)
            if sizerank != -1:
                self.parent.list.SetStringItem(index, sizerank, self.utility.formatedSize(self.requestedsize))

            oldcomplete = self.complete
            self.checkComplete()

            if self.abcengine and self.status != 'pause' and self.status != 'onhold':
                # Check for torrents switching from seeding to working
                if not self.complete and oldcomplete:
                    self.abcengine.stopSeedingTimers()
                    self.status = self.abcengine.btstatus = 'working'
                    self.abcengine.fin = False
                    self.termevt = self.prevtermevt = ''
                    self.parent.torrentavailable -= 1
                    self.parent.updateRunningTorrentCounters()
                # Check for ending forcerunning status in BitTornado
                if not self.unwantedfiles and self.abcengine.dow and self.abcengine.dow.forcerunning:
                    self.abcengine.dow.forcerunning = False
                    self.abcengine.dow._finished()
                self.infowin.updateSize()

            else:
                # Compute new progress according to files not set to "Never download"
                if self.requestedsize:
                    self.progress = downedbynotneversize / self.requestedsize * 100.
                    if self.progress > 100.:
                        self.progress = 100.
                    if self.progress < 0.:
                        self.progress = 0.
                else:
                    self.progress = 100.

                # Check new status for stopped torrent
                if self.status == 'stop' and self.complete:
                    # Torrent has turned completed or finished
                    if (self.uploadoption == "1" or self.uploadoption == "3") and self.seedingtimeleft == 0 \
                       or (self.uploadoption == "2" or self.uploadoption == "3") and self.ratioIsReached():
                        self.status = 'finished'
                        self.parent.torrentfinished += 1
                    else:
                        self.status = 'completed'
                    self.displayStatusOnly(index)
                    self.termevt = self.prevtermevt = time()
                    self.displayMessage(self.termEvtMsg(), index)
                    self.parent.torrentavailable += 1
                    self.parent.updateRunningTorrentCounters()
                elif (self.status == 'completed' or self.status == 'finished') and not self.complete:
                    # Torrent has turned stopped
                    if self.status == 'finished':
                        self.parent.torrentfinished -= 1
                    self.status = 'stop'
                    self.displayStatusOnly(index)
                    self.termevt = self.prevtermevt = ''
                    self.parent.torrentavailable -= 1
                    self.parent.updateRunningTorrentCounters()

                progress = self.formatedProgress()
                self.infowin.updateSize()
                self.infowin.totalprogress.SetLabel(progress)
                if self.detailwin:
                    self.detailwin.totalprogress.SetLabel(progress)
                progressrank = self.parent.guiman.getRankfromID(5)
                if progressrank != -1:
                    self.parent.list.SetStringItem(index, progressrank, progress)

                if self.abcparams['keepeta'] == "1":
                    # Display ETA
                    etarank = self.parent.guiman.getRankfromID(8)
                    if etarank != -1:
                        if self.status == 'finished':
                            self.parent.list.SetStringItem(index, etarank, "")
                        elif self.complete:
                            if self.uploadoption == "0":
                                self.parent.list.SetStringItem(index, etarank, self.localize('etaS') + "     oo")
                            elif self.uploadoption == "1":
                                self.parent.list.SetStringItem(index, etarank, self.localize('etaS') + " " + self.utility.eta_value(self.seedingtimeleft))
                            else:
                                self.parent.list.SetStringItem(index, etarank, self.localize('etaS') + "      ?")
                        else:
                            self.parent.list.SetStringItem(index, etarank, self.localize('etaD') + "      ?")

        # Write priorities to disk
        for i in xrange(self.filesnumber):
            if self.filespriority[i] != 1:
                break
        if self.filespriority[i] == 1:
            self.filespriority = None
            try:
                remove(self.src + '.pri')
            except:
                pass
        else:
            self.writeFilePriorities()

    def getResponse(self):
        if self.abcengine and self.abcengine.dow:
            metainfo = self.abcengine.dow.getResponse()
            return metainfo
        try:
            metainfo_file = open(self.src, 'rb')
            metainfo = bdecode(metainfo_file.read(), sloppy = 1)
        except:
            metainfo = None
        try:
            metainfo_file.close()
        except:
            pass
        return metainfo

    def getFinalDest(self):
        return join(self.utility.completePath(self.movefolder), self.name)

    def canWriteToDisk(self, onstart):
        drive = splitdrive(self.utility.completePath(self.dest))[0]
        try:
            freespace = GetDiskFreeSpaceEx(drive + '\\')[0]
        except:
            return False, self.localize('error') + ' : ' + self.localize('cantaccessdisk')
        if onstart:
            alreadyallocated = self.alreadyAllocated()
            self.totallyallocated = (alreadyallocated == self.requestedsize)
        if not self.totallyallocated:
            diskfullth = 1048576 * int(self.abcparams['diskfullth'])
            diskfull = False
            if onstart:
                if freespace < self.requestedsize - alreadyallocated + diskfullth:
                    diskfull = True
            elif freespace < diskfullth:
                diskfull = True
            if diskfull:
                message = self.localize('error') + ' : ' + self.localize('diskfull')
                if sys.platform == 'win32' and drive:
                    message += ' (' + drive + ')'
                return False, message
        return True, ''

    def alreadyAllocated(self):
        allocated = 0
        dest = self.utility.completePath(self.dest)
        if self.singlefile:
            if exists(dest):
                allocated = getsize(dest)
        else:
            try:
                info = self.getResponse()['info']
                i = 0
                for f in self.getFileNames():
                    if self.filespriority[i] != -1:
                        filename = join(dest, f)
                        if exists(filename):
                            allocated += getsize(filename)
                    i += 1
            except:
                allocated = 0
        return allocated

    def partialMarker(self):
        if self.unwantedfiles:
            return self.utility.partialmarker
        return ''

    def activityMarker(self):
        if self.activity == 1:
            return self.utility.activitymarker
        if self.activity == 2:
            return self.utility.standbymarker
        return self.utility.inactivitymarker

    def writeFileProgress(self, update = True, resetunwanted = False):
        if not self.singlefile and self.abcengine and (not self.complete or self.unwantedfiles):
            if update:
                self.abcengine.reportProgress()
            if self.filesprogress is not None:
                if resetunwanted and self.complete:
                    # Reset progress to 0 for unwanted files
                    if self.infowin is not None:
                        progresscol = 3 + self.infowin.originalfilenamesshown
                    for i in xrange(self.filesnumber):
                        if self.filespriority[i] == -1:
                            self.filesprogress[i] = '0'
                            if self.infowin is not None:
                                self.infowin.filelist.SetStringItem(i, progresscol, "0%")
                # Save progress in file
                try:
                    progressfile = open(self.src + '.pro', "w")
                    progressfile.writelines([p + '\n' for p in self.filesprogress])
                    progressfile.close()
                except:
                    pass

    def readFileProgress(self):
        try:
            progressfile = open(self.src + '.pro', "r")
        except:
            self.filesprogress = None
        else:
            self.filesprogress = []
            while True:
                templine = progressfile.readline()
                if templine == "" or templine == "\n":
                    break
                else:
                    self.filesprogress.append(templine[:-1].rstrip('%'))
                    # Drop '%' last character to match old format
            progressfile.close()
            if len(self.filesprogress) != self.filesnumber:
                # Corrupted progress file
                self.filesprogress = None
                remove(self.src + '.pro')

    def updateName(self, name, index):
        # Update torrent name
        self.name = name
        # Update name in list
        rank = self.parent.guiman.getRankfromID(4)
        if rank != -1:
            self.parent.list.SetStringItem(index, rank, name)
        # Update name in details and advanced details windows
        if self.infowin is not None:
            self.infowin.updateTorrentName()
        if self.detailwin is not None:
            self.detailwin.updateTorrentName()

    def updateDestination(self, dest, index = None):
        if index is None:
            try:
                index = self.parent.proctab.index(self)
            except:
                return
        self.dest = dest
        if self.magnet != 1:
            rank = self.parent.guiman.getRankfromID(26)
            if rank != -1:
                self.parent.list.SetStringItem(index, rank, dest)
            if self.infowin is not None:
                self.infowin.updateDestination()

    def destIsMove(self):
        # True if dest folder is the same as move folder
        return (normpath(self.utility.completePath(self.movefolder)).lower() == normpath(split(self.utility.completePath(self.dest))[0]).lower())

    def ratioIsReached(self):
        if self.abcengine:
            return (self.abcengine.shareratio >= float(self.uploadratio))
        if self.downsize > 0:
            shareratio = self.upsize / self.downsize * 100
        else:
            if self.requestedsize <= 0:
                shareratio = 999.9
            elif self.progress <= 0:
                shareratio = 0.
            else:
                shareratio = self.upsize / self.requestedsize / self.progress * 10000
        return shareratio >= float(self.uploadratio)

    def getOriginalFileNames(self, filesinfo):
        filesname = []
        for f in filesinfo:
            filename = u''
            if f.has_key('path.utf-8'):
                pathkey = 'path.utf-8'
            else:
                pathkey = 'path'
            for item in f[pathkey]:
                try:
                    filename = join(filename, item.decode('utf_8'))
                except:
                    filename = join(filename, item.decode(sysencoding))
            filesname.append(filename)
        return filesname

    def getFileNames(self, filesinfo = None):
        if self.filesname is None:
            if filesinfo is None:
                filesinfo = self.getResponse()['info']['files']
            return self.getOriginalFileNames(filesinfo)
        return self.filesname

    def deleteUnwanted(self):
        # Delete unwanted files and dir tree for a stopped multifile torrent
        if not self.unwantedfiles:
            return
        removedest = self.utility.completePath(self.dest)
        parentdir = split(removedest)[0]
        i = 0
        for filename in self.getFileNames():
            if self.filespriority[i] == -1:
                destname = join(removedest, filename)
                try:
                    remove(destname)
                except:
                    pass
                dir = split(destname)[0]
                try:
                    while not listdir(dir) and dir != parentdir:
                        rmdir(dir)
                        dir = split(dir)[0]
                except:
                    pass
            i += 1

    def resetFileNameStatus(self, files = None, mask = 0):
        # Reset bits of file name status for files according to mask
        if files is None:
            files = xrange(self.filesnumber)
        if self.filenamestatus:
            for i in files:
                self.filenamestatus[i] = self.filenamestatus[i] & mask
            if any(self.filenamestatus):
                self.writeFileNameStatus()
            else:
                self.filenamestatus = None
                try:
                    remove(self.src + '.fns')
                except:
                    pass
            self.infowin.displayFileNameStatus(files)

    def renameFiles(self, files, filesname, newname = None, filesinfo = None, caller = ''):
        # Note that when called from abcmetainfoframe, newname is None because it is
        # manually entered after checking if there's really something to rename
        if filesinfo is None:
            filesinfo = self.getResponse()['info']['files']
        if len(files) == 1:
            editable = filesname[files[0]]
            editablefordialog = editable
        else:
            editable = split(commonprefix([filesname[i] for i in files]))[0]
            editablefordialog = editable + '\\'
        if not editable:
            if caller != 'web':
                dlg = wx.MessageDialog(self.infowin, self.localize('errornothingtorename'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            return 'Nothing to rename'
        if newname is None:
            dialog = wx.TextEntryDialog(self.infowin,
                                        self.localize('editpath'),
                                        self.localize('editpath_short'),
                                        defaultValue = editablefordialog,
                                        style = wx.OK | wx.CANCEL)
            result = dialog.ShowModal()
            edited = dialog.GetValue()
            dialog.Destroy()
        else:
            edited = newname
        if edited and edited[0] == '\\':
            edited = edited[1:]
        if len(files) > 1 and edited and edited[-1] == '\\':
            edited = edited[:-1]
        if (caller != 'web' and result != wx.ID_OK) or edited.lower() == editable.lower():
            return 'Nothing to rename'
        if len(files) == 1 and not split(edited)[1]:
            if caller != 'web':
                dlg = wx.MessageDialog(self.infowin, self.localize('errorfilerenameempty'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            return 'The new file name is empty'
        # Check if path is relative
        if not self.utility.isPathRelative(edited):
            if caller != 'web':
                dlg = wx.MessageDialog(self.infowin, self.localize('errorfilerenamerelative'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            return 'The new path must be relative'
        # Check that file or folder name is valid
        if caller == 'web':
            parent = None
        else:
            parent = self.infowin
        if not self.utility.checkWinPath(parent, edited):
            return 'Invalid path name'

        # Check for file and folder name conflicts
        filetree = FileTree(filesname, self.dest, self.utility)
        errorfileconflict = False
        if len(files) == 1:
            if filetree.hasDir(edited):
                errorfileconflict = True
            else:
                item = edited
                while item:
                    if filetree.hasFile(item):
                        errorfileconflict = True
                        break
                    item = split(item)[0]
        else:
            item = edited
            while item:
                if filetree.hasFile(item):
                    errorfileconflict = True
                    break
                item = split(item)[0]
            if not errorfileconflict:
                editablelength = len(editable)
                for i in files:
                    if filesname[i].startswith(editable):
                        newname = edited + filesname[i][editablelength:]
                        if newname[0] == '\\':
                            newname = newname[1:]
                        if filetree.hasDir(newname):
                            errorfileconflict = True
                            break
                        item = newname
                        while item:
                            if filetree.hasFile(item):
                                errorfileconflict = True
                                break
                            item = split(item)[0]
                        if errorfileconflict:
                            break
        if errorfileconflict:
            if caller != 'web':
                dlg = wx.MessageDialog(self.infowin, self.localize('errorfileconflict'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            return 'Some of the new path names already exist'

        # Rename files and folders
        newnames = []
        if len(files) == 1:
            newnames.append(edited)
        else:
            editablelength = len(editable)
            destlen = len(self.utility.completePath(self.dest))
            for i in files:
                if filesname[i].startswith(editablefordialog):
                    newname = edited + filesname[i][editablelength:]
                    if newname[0] == '\\':
                        newname = newname[1:]
                    if len(newname) > 259 or len(split(newname)[0]) > 244:
                        if caller != 'web':
                            dlg = wx.MessageDialog(self.infowin, self.localize('errortoolongpath'),
                                                   self.localize('abcokcerror'), wx.ICON_ERROR)
                            dlg.ShowModal()
                            dlg.Destroy()
                        return 'Too long path name'
                    if len(newname) + destlen > 258 or len(split(newname)[0]) + destlen > 246:
                        if caller != 'web':
                            dlg = wx.MessageDialog(self.infowin, self.localize('errortoolongdest'),
                                                   self.localize('abcokcerror'), wx.ICON_ERROR)
                            dlg.ShowModal()
                            dlg.Destroy()
                        return 'Too long destination path name'
                    newnames.append(newname)

        # Reset file name status
        # Special case when renaming made two duplicate disappear : the renamed files and another one not selected
        for renamed in files:
            duplicate = []
            for f in xrange(self.filesnumber):
                if f not in files and filesname[f] == filesname[renamed]:
                    duplicate.append(f)
                    if len(duplicate) == 2:
                        break
            if len(duplicate) == 1:
                files.append(f)

        status = self.renameData(filesname, files, newnames, caller = caller)

        self.resetFileNameStatus(files)

        if filesname == self.getOriginalFileNames(filesinfo):
            if self.filesname is not None:
                self.filesname = None
                try:
                    remove(self.src + '.nam')
                except:
                    pass
        else:
            self.filesname = filesname
            self.writeFileNames()

        return status

    def renameData(self, filesname, files, newnames, caller = ''):
        # Rename files in torrent and already received data
        destfolder = self.utility.completePath(self.dest)
        dummyname = join(split(destfolder)[0], 'dummy')
        try:
            file(dummyname, 'w').close()
        except:
            pass
        error = False
        for f in xrange(len(files)):
            oldname = join(destfolder, filesname[files[f]])
            filesname[files[f]] = newnames[f]
            newname = join(destfolder, newnames[f])
            if newname != oldname and exists(oldname):
                try:
                    renames(oldname, newname)
                except:
                    error = True
        try:
            remove(dummyname)
        except:
            pass
        if error:
            if caller != 'web':
                dlg = wx.MessageDialog(self.infowin, self.localize('errordatarename'),
                                       self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            return 'Unable to rename some already received data'

    def resetCompleteStatus(self):
        self.complete = False

    def checkForReseed(self):
        # Test if a torrent has all its files downloaded and with the right size
        # so that reseed may or may not be started
        if self.singlefile:
            dest = self.utility.completePath(self.dest)
            if not exists(dest) or isdir(dest) or getsize(dest) != self.totalsize:
                return False
        else:
            filesizes = []
            try:
                filesinfo = self.getResponse()['info']['files']
            except:
                return False
            for info in filesinfo:
                filesizes.append(info['length'])
            i = 0
            for f in self.getFileNames(filesinfo):
                f = join(self.utility.completePath(self.dest), f)
                if not exists(f) or isdir(f) or getsize(f) != filesizes[i]:
                    return False
                i += 1
        return True

    def updateLimiters(self, value):
        # Update connections limiters counters by value
        if self.label not in self.parent.templates:
            # To deal with possible torrents with template labels without templates from older ABC_OKC versions
            # In current release a template can't be deleted as long as a torrent is assigned to it
            return False
        templimiters = self.parent.templates[self.label].conlimiters
        if self.complete and not self.unwantedfiles:
            # Seeding
            for limiter in templimiters:
                conlimiter = self.parent.conlimiters[limiter]
                conlimiter.nbseeding += value
                if conlimiter.nbseeding < 0:
                    conlimiter.nbseeding = 0
                #print self.name, ':', self.label, ':', limiter, ': s =', conlimiter.nbseeding, ': w =', conlimiter.nbworking, ': o =', conlimiter.nbseeding + conlimiter.nbworking
        else:
            # Working
            for limiter in templimiters:
                conlimiter = self.parent.conlimiters[limiter]
                conlimiter.nbworking += value
                if conlimiter.nbworking < 0:
                    conlimiter.nbworking = 0
                #print self.name, ':', self.label, ':', limiter, ': s =', conlimiter.nbseeding, ': w =', conlimiter.nbworking, ': o =', conlimiter.nbseeding + conlimiter.nbworking
        #if templimiters:
        #    return True
        #return False

    def applyRealCompleteStatus(self):
        # A not partial torrent that was declared as complete before starting up is first counted as seeding in the limiters counters.
        # This is done this way because always counting the torrents as "working" at start up in the limiters counters would have a consequence on
        # when the max of working would be reached, and it could trigger torrents from other templates farther in the list.
        # But when the checking at starting up fails meaning that the torrent was in fact uncomplete, the counters must be updated.
        # The contrary is also true : A not partial torrent started as uncomplete can reveal complete after a check ; counters must be updated here too.
        if not self.unwantedfiles:
            # Resync connections limiters counters
            if self.label not in self.parent.templates:
                return
            templimiters = self.parent.templates[self.label].conlimiters
            if self.abcengine.btcomplete:
                if not self.abcengine.alreadycomplete:
                    for limiter in templimiters:
                        conlimiter = self.parent.conlimiters[limiter]
                        conlimiter.nbseeding += 1
                        conlimiter.nbworking -= 1
                        if conlimiter.nbworking < 0:
                            conlimiter.nbworking = 0
                        #print self.name, ':', self.label, ':', limiter, ': s =', conlimiter.nbseeding, ': w =', conlimiter.nbworking, ': o =', conlimiter.nbseeding + conlimiter.nbworking
            elif self.abcengine.alreadycomplete:
                for limiter in templimiters:
                    conlimiter = self.parent.conlimiters[limiter]
                    conlimiter.nbseeding -= 1
                    if conlimiter.nbseeding < 0:
                        conlimiter.nbseeding = 0
                    conlimiter.nbworking += 1
                    #print self.name, ':', self.label, ':', limiter, ': s =', conlimiter.nbseeding, ': w =', conlimiter.nbworking, ': o =', conlimiter.nbseeding + conlimiter.nbworking
            if not self.abcengine.btcomplete and self.abcengine.alreadycomplete:
                # Torrent is not complete but alreadycomplete was wrongly set True ; reset alreadycomplete to real value so that next
                # running of procCOMPLETED gets this right value. 
                self.abcengine.alreadycomplete = False

    def updateLimitersWhenSeeding(self):
        # Update connections limiters when the torrent turns seeding
        if self.unwantedfiles or self.label not in self.parent.templates:
            return
        templimiters = self.parent.templates[self.label].conlimiters
        for limiter in templimiters:
            conlimiter = self.parent.conlimiters[limiter]
            conlimiter.nbworking -= 1
            if conlimiter.nbworking < 0:
                conlimiter.nbworking = 0
            conlimiter.nbseeding += 1
            #print self.name, ':', self.label, ':', limiter, ': s =', conlimiter.nbseeding, ': w =', conlimiter.nbworking, ': o =', conlimiter.nbseeding + conlimiter.nbworking

    def isNotLimited(self):
        # Returns True if not limited by connections limiters
        if self.label not in self.parent.templates:
            return True
        templimiters = self.parent.templates[self.label].conlimiters
        # Check number of connected
        for limiter in templimiters:
            conlimiter = self.parent.conlimiters[limiter]
            if conlimiter.maxoverall and conlimiter.nbseeding + conlimiter.nbworking >= conlimiter.maxoverall:
                return False
        # Check number of seeding and working
        if self.complete and not self.unwantedfiles:
            for limiter in templimiters:
                conlimiter = self.parent.conlimiters[limiter]
                if conlimiter.maxseeding and conlimiter.nbseeding >= conlimiter.maxseeding:
                    return False
        else:
            for limiter in templimiters:
                conlimiter = self.parent.conlimiters[limiter]
                if conlimiter.maxworking and conlimiter.nbworking >= conlimiter.maxworking:
                    return False
            # Check number of completed prevented from seeding (locked completed)
            # A locked completed is a completed torrent limited by the same limiters as the current torrent
            # and with a greater priority than the torrent's one, and that could not be started because
            # it was limited in upload (that is the number of seeds of one of its limiters had reached its max seeding value)
            if self.abcparams['lockedcompletedcheck'] == '1':
                sbqueued = (self.abcparams['sbqueued'] == '1')
                torindex = self.parent.proctab.index(self)
                self.parent.conlimitermanager.resetLockedCompletedCounters()
                for i in xrange(len(self.parent.proctab)):
                    t = self.parent.proctab[i]
                    if t.complete and not t.unwantedfiles \
                       and (t.status == 'queue' or sbqueued and t.status == 'standby') \
                       and (i < torindex and t.priority <= self.priority or i > torindex and t.priority < self.priority) \
                       and t.label in self.parent.templates:
                        templims = self.parent.templates[t.label].conlimiters
                        for limiter in templims:
                            conlimiter = self.parent.conlimiters[limiter]
                            if limiter in templimiters and conlimiter.maxseeding and conlimiter.nbseeding >= conlimiter.maxseeding:
                                conlimiter.nblockedcompleted += 1
                lockedcompletedmax = int(self.abcparams['lockedcompletedmax'])
                for limiter in templimiters:
                    conlimiter = self.parent.conlimiters[limiter]
                    if conlimiter.nblockedcompleted >= lockedcompletedmax:
                        return False
        return True

    def isOverSeedingLimit(self):
        # Returns True is torrent connections counters are over the max of seeding for at least one limiter
        if self.unwantedfiles or not self.complete or self.label not in self.parent.templates:
            return False
        templimiters = self.parent.templates[self.label].conlimiters
        for limiter in templimiters:
            conlimiter = self.parent.conlimiters[limiter]
            if conlimiter.maxseeding and conlimiter.nbseeding > conlimiter.maxseeding:
                return True
        return False

    def updateScrapeData(self, index, seed, peer, autoscraping, message, oldmsg = None):
        self.seed = seed
        self.peer = peer
        self.scrapeisfresh = True
        self.scrapedonce = True
        if self.abcengine:
            newseed = self.abcengine.numseedtext[:self.abcengine.numseedtext.index('(') + 1] + seed + ')'
            newpeer = self.abcengine.numpeertext[:self.abcengine.numpeertext.index('(') + 1] + peer + ')'
        else:
            newseed = '0 (' + seed + ')'
            newpeer = '0 (' + peer + ')'
        rank = self.parent.guiman.getRankfromID(14)
        if rank != -1:
            self.parent.list.SetStringItem(index, rank, newseed)
        rank = self.parent.guiman.getRankfromID(15)
        if rank != -1:
            self.parent.list.SetStringItem(index, rank, newpeer)

        # Display in advanced details window
        if self.detailwin is not None:
            self.detailwin.numseed.SetLabel(newseed)
            self.detailwin.numpeer.SetLabel(newpeer)

        if message is not None and not autoscraping:
            self.displayMessage(message, index, oldmsg = oldmsg)

    def termEvtMsg(self):
        if self.termevt:
            if type(self.termevt) == FloatType:
                if self.status == 'finished':
                    return strftime(self.utility.timestamp + ' -', localtime(self.termevt)) + self.localize('torisfinished')
                return strftime(self.utility.timestamp + ' -', localtime(self.termevt)) + self.localize('toriscompleted')
            return self.termevt
        return ''

    def getTracker(self):
        if self.exttracker:
            return self.exttrackerurl
        return self.defaulttracker

    def reannounce(self, tracker = None, url = None):
        if self.abcengine.reannouncelast == 0 or clock() - self.abcengine.reannouncelast > 60:
            self.abcengine.reannouncelast = clock()
            if self.abcengine.dow:
                self.abcengine.dow.reannounce(tracker = tracker, special = url)

    def initSeedingTimeLeft(self):
        # Computes default seeding time left for a torrent when there is no abcengine
        if self.uploadoption == '1':
            self.seedingtimeleft = max(self.uploadtimemax - self.seedingtime, 0)
        elif (self.uploadoption == '2' or self.uploadoption == '3') and self.ratioIsReached():
            self.seedingtimeleft = 0
        else:
            # Set to 100 days, equivalent to infinite
            self.seedingtimeleft = 8640000

    def incDecPriority(self, direction, index, priorank = None):
        if priorank is None:
            priorank = self.parent.guiman.getRankfromID(7)
        newprio = self.priority - direction
        if newprio > 4 or newprio < 0:
            return
        self.priority = newprio
        if priorank != -1:
            self.parent.list.SetStringItem(index, priorank, self.utility.priorities_s[newprio])

    def checkFileNames(self, parent, filesinfo = None, src = ''):
        # Try to fix characters and check length in items from file path
        filesname = self.getFileNames(filesinfo)
        torrentfiletree = FileTree(filesname, self.dest, self.utility, checkcase = True)
        fixed, self.filenamestatus = torrentfiletree.fixWinNames()
        if fixed:
            # Set new file names in torrent
            filesname = self.filesname = torrentfiletree.getFileNames()
            self.writeFileNames()
        # Check for total length of path name and path + file name of each file
        destlen = len(self.utility.completePath(self.dest))
        i = 0
        for f in filesname:
            # Limits for Windows : path : 247 char ; path + file : 259 char (+ NULL)
            # Don't forget additional '\' between dest and f
            if len(f) > 259 or len(split(f)[0]) > 244:
                # Too long name inside the torrent
                self.filenamestatus[i] |= 4
            if len(f) + destlen > 258 or len(split(f)[0]) + destlen > 246:
                # Too long name including destination path
                self.filenamestatus[i] |= 8
            i += 1

        # Compute and display results sum up
        checkstatus = 0
        for status in self.filenamestatus:
            checkstatus |= status
        if checkstatus:
            self.writeFileNameStatus()
            if src != 'web' and src != 'scan' and src != 'rss':
                sumupmessage = self.localize('filenamescheckresultprefix') + self.name \
                               + '\n' + self.localize('cantfixnameintorrent0')
                # Some automatic fixing failed
                if checkstatus & 1:
                    sumupmessage += self.localize('cantfixnameintorrent1')
                if checkstatus & 2:
                    sumupmessage += self.localize('cantfixnameintorrent2')
                if checkstatus & 4:
                    sumupmessage += self.localize('cantfixnameintorrent3')
                elif checkstatus & 8:
                    sumupmessage += self.localize('cantfixnameintorrent4')
                if checkstatus & 16:
                    sumupmessage += self.localize('cantfixnameintorrent5')
                # Show checks results sum up
                dlg = FileNamesCheckResultDialog(parent, -1, self.localize('filenamescheckresult'),
                                                 sumupmessage, fromdetailswin = (self.infowin is not None))
                self.parent.frame.restore()
                result = dlg.ShowModal()
                x, y = dlg.GetPositionTuple()
                self.utility.abcparams['filenamescheckresdlgx'], self.utility.abcparams['filenamescheckresdlgy'] = str(x), str(y)
                dlg.Destroy()
                showdetails = (result == wx.ID_OK)
            else:
                showdetails = False
        else:
            self.filenamestatus = None
            showdetails = False

        return checkstatus, showdetails

    def setDefaultTracker(self, deftracker):
        if self.infowin:
            if self.infowin.defaulttrackerindex != -1:
                self.infowin.trackerlist.SetStringItem(self.infowin.defaulttrackerindex, 1, '')
            self.infowin.markDefaultTracker(deftracker)
        self.defaulttracker = deftracker

    def updateData(self, src, dest, singlefile, name, totalsize, filesnumber, private, magnet):
        self.src = src
        self.dest = dest
        self.singlefile = singlefile
        self.name = name
        self.totalsize = totalsize
        self.requestedsize = float(totalsize)
        self.progress = 0.0
        self.filesnumber = filesnumber
        self.private = private
        self.magnet = magnet
        if self.abcengine:
            if self.abcengine.dow:
                self.abcengine.dow.updateData(src, dest, private)
            self.abcengine.statistics = None
